# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class SlideShapeTreesDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'slide_id': 'str',
        'slide': 'SlideSlidesDetails',
        'group_elements': 'list[SlideGroupElementsDetails]',
        'group_element_id': 'str',
        'group_element': 'SlideGroupElementsDetails',
        'hidden': 'bool',
        'title': 'str',
        'ooxml_id': 'int',
        'svg_blob_url': 'str',
        'base_element_blob_url': 'str',
        'changed_base_element_blob_url': 'str',
        'package_uri': 'str',
        'name': 'str',
        'id': 'str',
        'date_created': 'datetime',
        'user_created': 'str',
        'date_modified': 'datetime',
        'user_modified': 'str'
    }

    attribute_map = {
        'slide_id': 'slideId',
        'slide': 'slide',
        'group_elements': 'groupElements',
        'group_element_id': 'groupElementId',
        'group_element': 'groupElement',
        'hidden': 'hidden',
        'title': 'title',
        'ooxml_id': 'ooxmlId',
        'svg_blob_url': 'svgBlobUrl',
        'base_element_blob_url': 'baseElementBlobUrl',
        'changed_base_element_blob_url': 'changedBaseElementBlobUrl',
        'package_uri': 'packageUri',
        'name': 'name',
        'id': 'id',
        'date_created': 'dateCreated',
        'user_created': 'userCreated',
        'date_modified': 'dateModified',
        'user_modified': 'userModified'
    }

    def __init__(self, slide_id=None, slide=None, group_elements=None, group_element_id=None, group_element=None, hidden=None, title=None, ooxml_id=None, svg_blob_url=None, base_element_blob_url=None, changed_base_element_blob_url=None, package_uri=None, name=None, id=None, date_created=None, user_created=None, date_modified=None, user_modified=None):  # noqa: E501
        """SlideShapeTreesDetails - a model defined in OpenAPI"""  # noqa: E501

        self._slide_id = None
        self._slide = None
        self._group_elements = None
        self._group_element_id = None
        self._group_element = None
        self._hidden = None
        self._title = None
        self._ooxml_id = None
        self._svg_blob_url = None
        self._base_element_blob_url = None
        self._changed_base_element_blob_url = None
        self._package_uri = None
        self._name = None
        self._id = None
        self._date_created = None
        self._user_created = None
        self._date_modified = None
        self._user_modified = None
        self.discriminator = None

        self.slide_id = slide_id
        if slide is not None:
            self.slide = slide
        self.group_elements = group_elements
        self.group_element_id = group_element_id
        if group_element is not None:
            self.group_element = group_element
        if hidden is not None:
            self.hidden = hidden
        self.title = title
        if ooxml_id is not None:
            self.ooxml_id = ooxml_id
        self.svg_blob_url = svg_blob_url
        self.base_element_blob_url = base_element_blob_url
        self.changed_base_element_blob_url = changed_base_element_blob_url
        self.package_uri = package_uri
        self.name = name
        if id is not None:
            self.id = id
        if date_created is not None:
            self.date_created = date_created
        if user_created is not None:
            self.user_created = user_created
        if date_modified is not None:
            self.date_modified = date_modified
        if user_modified is not None:
            self.user_modified = user_modified

    @property
    def slide_id(self):
        """Gets the slide_id of this SlideShapeTreesDetails.  # noqa: E501


        :return: The slide_id of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._slide_id

    @slide_id.setter
    def slide_id(self, slide_id):
        """Sets the slide_id of this SlideShapeTreesDetails.


        :param slide_id: The slide_id of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._slide_id = slide_id

    @property
    def slide(self):
        """Gets the slide of this SlideShapeTreesDetails.  # noqa: E501


        :return: The slide of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: SlideSlidesDetails
        """
        return self._slide

    @slide.setter
    def slide(self, slide):
        """Sets the slide of this SlideShapeTreesDetails.


        :param slide: The slide of this SlideShapeTreesDetails.  # noqa: E501
        :type: SlideSlidesDetails
        """

        self._slide = slide

    @property
    def group_elements(self):
        """Gets the group_elements of this SlideShapeTreesDetails.  # noqa: E501


        :return: The group_elements of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: list[SlideGroupElementsDetails]
        """
        return self._group_elements

    @group_elements.setter
    def group_elements(self, group_elements):
        """Sets the group_elements of this SlideShapeTreesDetails.


        :param group_elements: The group_elements of this SlideShapeTreesDetails.  # noqa: E501
        :type: list[SlideGroupElementsDetails]
        """

        self._group_elements = group_elements

    @property
    def group_element_id(self):
        """Gets the group_element_id of this SlideShapeTreesDetails.  # noqa: E501


        :return: The group_element_id of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._group_element_id

    @group_element_id.setter
    def group_element_id(self, group_element_id):
        """Sets the group_element_id of this SlideShapeTreesDetails.


        :param group_element_id: The group_element_id of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._group_element_id = group_element_id

    @property
    def group_element(self):
        """Gets the group_element of this SlideShapeTreesDetails.  # noqa: E501


        :return: The group_element of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: SlideGroupElementsDetails
        """
        return self._group_element

    @group_element.setter
    def group_element(self, group_element):
        """Sets the group_element of this SlideShapeTreesDetails.


        :param group_element: The group_element of this SlideShapeTreesDetails.  # noqa: E501
        :type: SlideGroupElementsDetails
        """

        self._group_element = group_element

    @property
    def hidden(self):
        """Gets the hidden of this SlideShapeTreesDetails.  # noqa: E501


        :return: The hidden of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: bool
        """
        return self._hidden

    @hidden.setter
    def hidden(self, hidden):
        """Sets the hidden of this SlideShapeTreesDetails.


        :param hidden: The hidden of this SlideShapeTreesDetails.  # noqa: E501
        :type: bool
        """

        self._hidden = hidden

    @property
    def title(self):
        """Gets the title of this SlideShapeTreesDetails.  # noqa: E501


        :return: The title of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this SlideShapeTreesDetails.


        :param title: The title of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._title = title

    @property
    def ooxml_id(self):
        """Gets the ooxml_id of this SlideShapeTreesDetails.  # noqa: E501


        :return: The ooxml_id of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: int
        """
        return self._ooxml_id

    @ooxml_id.setter
    def ooxml_id(self, ooxml_id):
        """Sets the ooxml_id of this SlideShapeTreesDetails.


        :param ooxml_id: The ooxml_id of this SlideShapeTreesDetails.  # noqa: E501
        :type: int
        """

        self._ooxml_id = ooxml_id

    @property
    def svg_blob_url(self):
        """Gets the svg_blob_url of this SlideShapeTreesDetails.  # noqa: E501


        :return: The svg_blob_url of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._svg_blob_url

    @svg_blob_url.setter
    def svg_blob_url(self, svg_blob_url):
        """Sets the svg_blob_url of this SlideShapeTreesDetails.


        :param svg_blob_url: The svg_blob_url of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._svg_blob_url = svg_blob_url

    @property
    def base_element_blob_url(self):
        """Gets the base_element_blob_url of this SlideShapeTreesDetails.  # noqa: E501


        :return: The base_element_blob_url of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._base_element_blob_url

    @base_element_blob_url.setter
    def base_element_blob_url(self, base_element_blob_url):
        """Sets the base_element_blob_url of this SlideShapeTreesDetails.


        :param base_element_blob_url: The base_element_blob_url of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._base_element_blob_url = base_element_blob_url

    @property
    def changed_base_element_blob_url(self):
        """Gets the changed_base_element_blob_url of this SlideShapeTreesDetails.  # noqa: E501


        :return: The changed_base_element_blob_url of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._changed_base_element_blob_url

    @changed_base_element_blob_url.setter
    def changed_base_element_blob_url(self, changed_base_element_blob_url):
        """Sets the changed_base_element_blob_url of this SlideShapeTreesDetails.


        :param changed_base_element_blob_url: The changed_base_element_blob_url of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._changed_base_element_blob_url = changed_base_element_blob_url

    @property
    def package_uri(self):
        """Gets the package_uri of this SlideShapeTreesDetails.  # noqa: E501


        :return: The package_uri of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._package_uri

    @package_uri.setter
    def package_uri(self, package_uri):
        """Sets the package_uri of this SlideShapeTreesDetails.


        :param package_uri: The package_uri of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._package_uri = package_uri

    @property
    def name(self):
        """Gets the name of this SlideShapeTreesDetails.  # noqa: E501


        :return: The name of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this SlideShapeTreesDetails.


        :param name: The name of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def id(self):
        """Gets the id of this SlideShapeTreesDetails.  # noqa: E501


        :return: The id of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SlideShapeTreesDetails.


        :param id: The id of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def date_created(self):
        """Gets the date_created of this SlideShapeTreesDetails.  # noqa: E501


        :return: The date_created of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_created

    @date_created.setter
    def date_created(self, date_created):
        """Sets the date_created of this SlideShapeTreesDetails.


        :param date_created: The date_created of this SlideShapeTreesDetails.  # noqa: E501
        :type: datetime
        """

        self._date_created = date_created

    @property
    def user_created(self):
        """Gets the user_created of this SlideShapeTreesDetails.  # noqa: E501


        :return: The user_created of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_created

    @user_created.setter
    def user_created(self, user_created):
        """Sets the user_created of this SlideShapeTreesDetails.


        :param user_created: The user_created of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._user_created = user_created

    @property
    def date_modified(self):
        """Gets the date_modified of this SlideShapeTreesDetails.  # noqa: E501


        :return: The date_modified of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_modified

    @date_modified.setter
    def date_modified(self, date_modified):
        """Sets the date_modified of this SlideShapeTreesDetails.


        :param date_modified: The date_modified of this SlideShapeTreesDetails.  # noqa: E501
        :type: datetime
        """

        self._date_modified = date_modified

    @property
    def user_modified(self):
        """Gets the user_modified of this SlideShapeTreesDetails.  # noqa: E501


        :return: The user_modified of this SlideShapeTreesDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_modified

    @user_modified.setter
    def user_modified(self, user_modified):
        """Sets the user_modified of this SlideShapeTreesDetails.


        :param user_modified: The user_modified of this SlideShapeTreesDetails.  # noqa: E501
        :type: str
        """

        self._user_modified = user_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SlideShapeTreesDetails):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class SharedTextDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'paragraph_id': 'str',
        'paragraph': 'SharedParagraphDetails',
        'sequence': 'int',
        'raw_text': 'str',
        'color_solid_fills_id': 'str',
        'color_solid_fill': 'SharedSolidFillsDetails',
        'is_bold': 'bool',
        'is_italic': 'bool',
        'is_underline': 'bool',
        'font_size': 'int',
        'font': 'str',
        'is_theme_font': 'bool',
        'id': 'str',
        'date_created': 'datetime',
        'user_created': 'str',
        'date_modified': 'datetime',
        'user_modified': 'str'
    }

    attribute_map = {
        'paragraph_id': 'paragraphId',
        'paragraph': 'paragraph',
        'sequence': 'sequence',
        'raw_text': 'rawText',
        'color_solid_fills_id': 'colorSolidFillsId',
        'color_solid_fill': 'colorSolidFill',
        'is_bold': 'isBold',
        'is_italic': 'isItalic',
        'is_underline': 'isUnderline',
        'font_size': 'fontSize',
        'font': 'font',
        'is_theme_font': 'isThemeFont',
        'id': 'id',
        'date_created': 'dateCreated',
        'user_created': 'userCreated',
        'date_modified': 'dateModified',
        'user_modified': 'userModified'
    }

    def __init__(self, paragraph_id=None, paragraph=None, sequence=None, raw_text=None, color_solid_fills_id=None, color_solid_fill=None, is_bold=None, is_italic=None, is_underline=None, font_size=None, font=None, is_theme_font=None, id=None, date_created=None, user_created=None, date_modified=None, user_modified=None):  # noqa: E501
        """SharedTextDetails - a model defined in OpenAPI"""  # noqa: E501

        self._paragraph_id = None
        self._paragraph = None
        self._sequence = None
        self._raw_text = None
        self._color_solid_fills_id = None
        self._color_solid_fill = None
        self._is_bold = None
        self._is_italic = None
        self._is_underline = None
        self._font_size = None
        self._font = None
        self._is_theme_font = None
        self._id = None
        self._date_created = None
        self._user_created = None
        self._date_modified = None
        self._user_modified = None
        self.discriminator = None

        self.paragraph_id = paragraph_id
        if paragraph is not None:
            self.paragraph = paragraph
        if sequence is not None:
            self.sequence = sequence
        self.raw_text = raw_text
        self.color_solid_fills_id = color_solid_fills_id
        if color_solid_fill is not None:
            self.color_solid_fill = color_solid_fill
        if is_bold is not None:
            self.is_bold = is_bold
        if is_italic is not None:
            self.is_italic = is_italic
        if is_underline is not None:
            self.is_underline = is_underline
        self.font_size = font_size
        self.font = font
        if is_theme_font is not None:
            self.is_theme_font = is_theme_font
        if id is not None:
            self.id = id
        if date_created is not None:
            self.date_created = date_created
        if user_created is not None:
            self.user_created = user_created
        if date_modified is not None:
            self.date_modified = date_modified
        if user_modified is not None:
            self.user_modified = user_modified

    @property
    def paragraph_id(self):
        """Gets the paragraph_id of this SharedTextDetails.  # noqa: E501


        :return: The paragraph_id of this SharedTextDetails.  # noqa: E501
        :rtype: str
        """
        return self._paragraph_id

    @paragraph_id.setter
    def paragraph_id(self, paragraph_id):
        """Sets the paragraph_id of this SharedTextDetails.


        :param paragraph_id: The paragraph_id of this SharedTextDetails.  # noqa: E501
        :type: str
        """

        self._paragraph_id = paragraph_id

    @property
    def paragraph(self):
        """Gets the paragraph of this SharedTextDetails.  # noqa: E501


        :return: The paragraph of this SharedTextDetails.  # noqa: E501
        :rtype: SharedParagraphDetails
        """
        return self._paragraph

    @paragraph.setter
    def paragraph(self, paragraph):
        """Sets the paragraph of this SharedTextDetails.


        :param paragraph: The paragraph of this SharedTextDetails.  # noqa: E501
        :type: SharedParagraphDetails
        """

        self._paragraph = paragraph

    @property
    def sequence(self):
        """Gets the sequence of this SharedTextDetails.  # noqa: E501


        :return: The sequence of this SharedTextDetails.  # noqa: E501
        :rtype: int
        """
        return self._sequence

    @sequence.setter
    def sequence(self, sequence):
        """Sets the sequence of this SharedTextDetails.


        :param sequence: The sequence of this SharedTextDetails.  # noqa: E501
        :type: int
        """

        self._sequence = sequence

    @property
    def raw_text(self):
        """Gets the raw_text of this SharedTextDetails.  # noqa: E501


        :return: The raw_text of this SharedTextDetails.  # noqa: E501
        :rtype: str
        """
        return self._raw_text

    @raw_text.setter
    def raw_text(self, raw_text):
        """Sets the raw_text of this SharedTextDetails.


        :param raw_text: The raw_text of this SharedTextDetails.  # noqa: E501
        :type: str
        """

        self._raw_text = raw_text

    @property
    def color_solid_fills_id(self):
        """Gets the color_solid_fills_id of this SharedTextDetails.  # noqa: E501


        :return: The color_solid_fills_id of this SharedTextDetails.  # noqa: E501
        :rtype: str
        """
        return self._color_solid_fills_id

    @color_solid_fills_id.setter
    def color_solid_fills_id(self, color_solid_fills_id):
        """Sets the color_solid_fills_id of this SharedTextDetails.


        :param color_solid_fills_id: The color_solid_fills_id of this SharedTextDetails.  # noqa: E501
        :type: str
        """

        self._color_solid_fills_id = color_solid_fills_id

    @property
    def color_solid_fill(self):
        """Gets the color_solid_fill of this SharedTextDetails.  # noqa: E501


        :return: The color_solid_fill of this SharedTextDetails.  # noqa: E501
        :rtype: SharedSolidFillsDetails
        """
        return self._color_solid_fill

    @color_solid_fill.setter
    def color_solid_fill(self, color_solid_fill):
        """Sets the color_solid_fill of this SharedTextDetails.


        :param color_solid_fill: The color_solid_fill of this SharedTextDetails.  # noqa: E501
        :type: SharedSolidFillsDetails
        """

        self._color_solid_fill = color_solid_fill

    @property
    def is_bold(self):
        """Gets the is_bold of this SharedTextDetails.  # noqa: E501


        :return: The is_bold of this SharedTextDetails.  # noqa: E501
        :rtype: bool
        """
        return self._is_bold

    @is_bold.setter
    def is_bold(self, is_bold):
        """Sets the is_bold of this SharedTextDetails.


        :param is_bold: The is_bold of this SharedTextDetails.  # noqa: E501
        :type: bool
        """

        self._is_bold = is_bold

    @property
    def is_italic(self):
        """Gets the is_italic of this SharedTextDetails.  # noqa: E501


        :return: The is_italic of this SharedTextDetails.  # noqa: E501
        :rtype: bool
        """
        return self._is_italic

    @is_italic.setter
    def is_italic(self, is_italic):
        """Sets the is_italic of this SharedTextDetails.


        :param is_italic: The is_italic of this SharedTextDetails.  # noqa: E501
        :type: bool
        """

        self._is_italic = is_italic

    @property
    def is_underline(self):
        """Gets the is_underline of this SharedTextDetails.  # noqa: E501


        :return: The is_underline of this SharedTextDetails.  # noqa: E501
        :rtype: bool
        """
        return self._is_underline

    @is_underline.setter
    def is_underline(self, is_underline):
        """Sets the is_underline of this SharedTextDetails.


        :param is_underline: The is_underline of this SharedTextDetails.  # noqa: E501
        :type: bool
        """

        self._is_underline = is_underline

    @property
    def font_size(self):
        """Gets the font_size of this SharedTextDetails.  # noqa: E501


        :return: The font_size of this SharedTextDetails.  # noqa: E501
        :rtype: int
        """
        return self._font_size

    @font_size.setter
    def font_size(self, font_size):
        """Sets the font_size of this SharedTextDetails.


        :param font_size: The font_size of this SharedTextDetails.  # noqa: E501
        :type: int
        """

        self._font_size = font_size

    @property
    def font(self):
        """Gets the font of this SharedTextDetails.  # noqa: E501


        :return: The font of this SharedTextDetails.  # noqa: E501
        :rtype: str
        """
        return self._font

    @font.setter
    def font(self, font):
        """Sets the font of this SharedTextDetails.


        :param font: The font of this SharedTextDetails.  # noqa: E501
        :type: str
        """

        self._font = font

    @property
    def is_theme_font(self):
        """Gets the is_theme_font of this SharedTextDetails.  # noqa: E501


        :return: The is_theme_font of this SharedTextDetails.  # noqa: E501
        :rtype: bool
        """
        return self._is_theme_font

    @is_theme_font.setter
    def is_theme_font(self, is_theme_font):
        """Sets the is_theme_font of this SharedTextDetails.


        :param is_theme_font: The is_theme_font of this SharedTextDetails.  # noqa: E501
        :type: bool
        """

        self._is_theme_font = is_theme_font

    @property
    def id(self):
        """Gets the id of this SharedTextDetails.  # noqa: E501


        :return: The id of this SharedTextDetails.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SharedTextDetails.


        :param id: The id of this SharedTextDetails.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def date_created(self):
        """Gets the date_created of this SharedTextDetails.  # noqa: E501


        :return: The date_created of this SharedTextDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_created

    @date_created.setter
    def date_created(self, date_created):
        """Sets the date_created of this SharedTextDetails.


        :param date_created: The date_created of this SharedTextDetails.  # noqa: E501
        :type: datetime
        """

        self._date_created = date_created

    @property
    def user_created(self):
        """Gets the user_created of this SharedTextDetails.  # noqa: E501


        :return: The user_created of this SharedTextDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_created

    @user_created.setter
    def user_created(self, user_created):
        """Sets the user_created of this SharedTextDetails.


        :param user_created: The user_created of this SharedTextDetails.  # noqa: E501
        :type: str
        """

        self._user_created = user_created

    @property
    def date_modified(self):
        """Gets the date_modified of this SharedTextDetails.  # noqa: E501


        :return: The date_modified of this SharedTextDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_modified

    @date_modified.setter
    def date_modified(self, date_modified):
        """Sets the date_modified of this SharedTextDetails.


        :param date_modified: The date_modified of this SharedTextDetails.  # noqa: E501
        :type: datetime
        """

        self._date_modified = date_modified

    @property
    def user_modified(self):
        """Gets the user_modified of this SharedTextDetails.  # noqa: E501


        :return: The user_modified of this SharedTextDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_modified

    @user_modified.setter
    def user_modified(self, user_modified):
        """Sets the user_modified of this SharedTextDetails.


        :param user_modified: The user_modified of this SharedTextDetails.  # noqa: E501
        :type: str
        """

        self._user_modified = user_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SharedTextDetails):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import typing
import os
import sys
import json
import jsonpatch
import jsonpointer
import logging
import presalytics.client.api
import presalytics.lib.tools.story_tools
import presalytics.story.outline
import presalytics.story.components
if typing.TYPE_CHECKING:
    from presalytics.story.components import ComponentBase, WidgetBase
    from presalytics.story.outline import StoryOutline

logger = logging.getLogger(__name__)

def update_components(filename=None):
    autodiscover_paths = presalytics.COMPONENTS.autodiscover_paths
    dirname = os.path.dirname(filename)
    if os.path.exists(dirname) and dirname not in autodiscover_paths:
        autodiscover_paths.append(dirname)
    cwd = os.getcwd()
    abs_filedir = os.path.join(cwd, dirname)
    if os.path.exists(abs_filedir) and abs_filedir not in autodiscover_paths:
        autodiscover_paths.append(abs_filedir)
    if len(autodiscover_paths) > len(presalytics.COMPONENTS.autodiscover_paths):
        presalytics.COMPONENTS = presalytics.story.components.ComponentRegistry(autodiscover_paths=autodiscover_paths, 
                                                                                reserved_names=presalytics.CONFIG.get("RESERVED_NAMES", []))

def get_component(name, filename=None):
    inst: 'ComponentBase'

    if filename:
        update_components(filename=filename)            
    component_list = presalytics.COMPONENTS.find_instance(name)
    if len(component_list) == 0:
        message = "Could not find an instance with name '{0}' in the COMPONENTS registry".format(name)
        presalytics.COMPONENTS.raise_error(message)
    elif len(component_list) > 1:
        components_str = ", ".join(component_list)
        message = "Multiple matches for name in COMPONENTS registry.  Choose from: " + components_str
        presalytics.COMPONENTS.raise_error(message)
    elif len(component_list) == 1:
        return presalytics.COMPONENTS.get_instance(component_list[0])
    else:
        message = "Error searching the COMPONENTS registry"
        presalytics.COMPONENTS.raise_error(message)

def add_widget_instance(widget_name, outline, position=None, page_number=0, filename=None):
    inst: 'WidgetBase'
    outline: 'StoryOutline'

    inst = get_component(widget_name, filename=filename)
    widget = inst.serialize()
    if not page_number:
        page_number = 0
    if page_number > len(outline.pages):
        message = "The story has fewer than {} pages".format(page_number)
        raise presalytics.lib.exceptions.InvalidArgumentException(message=message)
    if not position:
        position = len(outline.pages[page_number].widgets)
    outline.pages[page_number].widgets.insert(position, widget)

    return outline

def remove_widget_by_name(widget_name, outline, page_number=None, filename=None):
    inst: 'WidgetBase'
    outline: 'StoryOutline'

    if page_number:
        start = page_number
        stop = page_number
    else:
        start = 0
        stop = len(outline.pages)
    
    for p in range(start, stop):
        page = outline.pages[p]
        for w in range(0, len(page.widgets)):
            if page.widgets[w].name == widget_name:
                page.widgets.pop(w)
    
    return outline



def create_from_instance(name, widget=False, page=False, filename=None) -> 'StoryOutline': #type: ignore
    """
    Creates a Story Outline from an instance name in `presalytics.COMPONENTS`
    """
    inst: 'ComponentBase'

    inst = get_component(name)
    if page:
        outline = presalytics.lib.tools.component_tools.create_outline_from_page(inst) #type: ignore
    elif widget:
        outline = presalytics.lib.tools.component_tools.create_outline_from_widget(inst) #type: ignore
    else:
        raise presalytics.lib.exceptions.InvalidArgumentException("Either the 'widget' or 'page' paramters must be true.")
    return outline



def push_outline(outline, username=None, password=None) -> 'StoryOutline':
    """
    Updates an outline in the Presalytics API Story service
    """
    client = presalytics.client.api.Client(username=username, password=password)
    if outline.story_id == "empty" or not outline.story_id:
        story = client.story.story_post({"outline": outline.dump()})
    else:
        old_story = client.story.story_id_get(outline.story_id)
        old_story.outline = outline.dump()
        story = client.story.story_id_outline_post(old_story.id, outline.dump())
    synced_outline_str = client.await_outline(outline.story_id)
    return presalytics.story.outline.StoryOutline.load(synced_outline_str)


def pull_outline(story_id, username=None, password=None) -> 'StoryOutline':
    """
    Retrives an outline from the Presalytics API Story service
    """
    client = presalytics.client.api.Client(username=username, password=password)
    if story_id == "empty" or not story_id:
        message = "The story_id must not be empty"
        raise presalytics.lib.exceptions.InvalidArgumentException(message)
    story = client.story.story_id_get(story_id)
    return presalytics.story.outline.StoryOutline.load(story.outline)


def delete_all_stories(username=None, password=None):
    """
    WARNING!!! 
    
    Delete all stories owned by this user

    BE CAREFUL!!!    
    """
    client = presalytics.client.api.Client(username=username, password=password)
    stories = client.story.story_get()
    ids = [s.id for s in stories]
    for id in ids:
        client.story.story_id_delete(id)


def delete_by_id(story_id, username=None, password=None):
    """
    Delete stories by Story Id
    """
    client = presalytics.client.api.Client(username=username, password=password)
    client.story.story_id_delete(story_id)

def share_story(story_id,
                emails=None, 
                user_ids=None, 
                username=None, 
                password=None,
                collaborator_type="viewer"):
    """
    Share stories with other users. Either by email or Presalytics API user Id.
    """
    client = presalytics.client.api.Client(username=username, password=password)
    if emails:
        email_list = emails.split(",")
        for e in email_list:
            body = {
                "user_email": e,
                "collaborator_type": collaborator_type,
                "user_id": ""
            }
            client.story.story_id_collaborators_post(story_id, body)
    if user_ids:
        user_id_list = user_ids.split(",")
        for _id in user_id_list:
            body = {
                "user_email": "",
                "collaborator_type": collaborator_type,
                "user_id": _id
            }
            client.story.story_id_collaborators_post(story_id, body)

def get_story(story_id):
    """
    returns Story metadata from the story API
    """
    client = presalytics.client.api.Client()
    return client.story.story_id_get(story_id, include_relationships=True)

def create_config_file(username, password=None, set_dict={}, overwrite=False):
    config_file = os.path.join(os.getcwd(), "config.py")
    if os.path.exists(config_file):
        if overwrite:
            os.remove(config_file)
        else:
            logger.error("Configuration file exists.  Use option [--overwrite] to overwrite.")
            raise ValueError
    dump = {"USERNAME": username}
    if password:
        dump.update({"PASSWORD": password})
    if len(set_dict.keys()) > 0:
        dump.update(set_dict)
    content = "PRESALYTICS = " + repr(dump)
    with open(config_file, 'w') as f:
        f.write(content)

def create_cron_target():
    if sys.platform.startswith("linux"):
        cron_command = "* * * * * cd {0} && {1} -m presalytics push".format(os.getcwd(), sys.executable)
        print("Place the following line in your cron file:")
        print(cron_command)
    elif sys.platform.startswith("win"):
        command = os.environ["VIRTUAL_ENV"] + "/Scripts/activate.bat && python -m presalytics push"
        filename = os.path.join(os.getcwd(), "story-push.bat")
        with open(filename, 'w') as f:
            f.write(command)
        print("A file named 'story-push.bat was just placed in the current working directory.  Add this file in the 'Windows Task Scheduler'.")
    else:
        logger.error("Unknown operating system.")
        raise ValueError

def apply_json_patch(outline, patch):
    """
    Allows users to apply json patches to thier Story Outlines.

    See https://python-json-patch.readthedocs.io/en/latest/index.html for more info.

    Parameters:
    ----------

    outline: presaltyics.story.outline.StoryOutline
        A Story Outline object

    patch: str or dict
        Either an RFC 6901 compliant string or a dictionary with op, path, and value keys.  The
        dictionary is for ease of use and will get parsed into a compliant string.
    """
    outline: 'StoryOutline'

    if isinstance(patch, dict):
        _json = json.dumps(patch)
        patch = '[{}]'.format(_json)
    outline_dict = outline.to_dict()
    try:
        new_dict = jsonpatch.apply_patch(outline_dict, patch)
    except (jsonpatch.JsonPatchException, jsonpointer.JsonPointerException) as ex:
        message = "Unable to apply Json patch: {}".format(ex.args[0])
        raise presalytics.lib.exceptions.InvalidArgumentException(message=message)
    new_outline = presalytics.story.outline.StoryOutline.deserialize(new_dict)
    return new_outline

def update_outline(outline, filename=None, message=None):
    """
    Updates the the outline from active instances in the workspace
    """
    if not message:
        message = "Automated update via presalytics.lib.tools.workflows.update_outline()"
    
    update_components(filename=filename)

    for p in range(0, len(outline.pages)):
        page = outline.pages[p]

        page_key = "page.{}.{}".format(page.kind, page.name)
        page_inst = presalytics.COMPONENTS.get_instance(page_key)
        if page_inst:
            outline.pages[p] = page_inst.serialize()
        
        for w in range(0, len(page.widgets)):
            widget = outline.pages[p].widgets[w]

            widget_key = "widget.{}.{}".format(widget.kind, widget.name)
            widget_inst = presalytics.COMPONENTS.get_instance(widget_key)
            if widget_inst:
                outline.pages[p].widgets[w] = widget_inst.serialize() 

    
    for t in range(0, len(outline.themes)):
        theme = outline.themes[t]

        theme_key = "theme.{}.{}".format(theme.kind, theme.name)
        theme_inst = presalytics.COMPONENTS.get_instance(theme_key)
        if theme_inst:
            outline.themes[t] = theme_inst.serialize()
    
    outline.info.revision_notes = message
    return outline


    
    


    


import logging
import logging.config
import os
import sys
import presalytics

default_log_path = os.getcwd() + os.path.sep + 'presalytics.log'

USE_LOGGER = False

def configure_logger(log_path=default_log_path, log_level='DEBUG', file_logger=True):
    logging.config.dictConfig({
        'version': 1,
        'formatters': {
            'default': {'format': '%(asctime)s - %(threadName)s - %(name)s - %(levelname)s - %(message)s'}
        },
        'handlers': {
            'console': {
                'level': log_level,
                'class': 'logging.StreamHandler',
                'formatter': 'default',
                'stream': 'ext://sys.stdout'
            },
        },
        'loggers': {
            'presalytics': {
                'level': log_level,
                'handlers': ['console']
            }
        },
        'disable_existing_loggers': False
    })

    if file_logger:
        logger = logging.getLogger('presalytics')
        log_dir = os.path.join(os.getcwd(), 'log')
        if not os.path.exists(log_dir):
            os.mkdir(log_dir)
        log_file = os.path.join(log_dir, 'presalytics.log')
        file_handler = logging.FileHandler(log_file)
        logger.addHandler(file_handler)
    
    USE_LOGGER = file_logger

def handle_exception(exc_type, exc_value, exc_traceback):
    """ Catches unhandled exceptions for logger """
    try:
        logger = logging.getLogger('presalytics.exception_hook')
        if USE_LOGGER:
            logger.error("--------------------------")
            logger.error("UNHANDLED EXCEPTION: FATAL")
            logger.error(msg="--------------------------", exc_info=(exc_type, exc_value, exc_traceback,))
            logger.error("--------------------------")
            logger.error("UNHANDLED EXCEPTION: FATAL")
            logger.error("--------------------------")
        else:
            logger.error(msg="An unhandled exception occured.", exc_info=(exc_type, exc_value, exc_traceback,))
    except Exception:
        pass
    return


# users can override the except hook, but most analysts aren't that skilled yet.
# Makes their scripts more verbose without having to think about it
sys.excepthook = handle_exception



"""
Module helps serialize and deserialize presalytics story outlines to/from json and yaml source data.
Classes in this module acts are interfaces between story outlines and `component` objects
"""
import json
import numpy as np
import base64
import yaml
import inspect
import datetime
import dateutil.parser
import uuid
import abc
import typing
import sys
import os
import semantic_version
import jsonschema
from presalytics.story.util import to_camel_case, to_snake_case
from presalytics.lib.exceptions import ValidationError

class OutlineEncoder(json.JSONEncoder):
    """
    Json encoder for `presalytics.story.outline.OutlineBase` objects
    """
    def default(self, obj):
        """
        Override method for deserializing objects that inherit from 
        `presalytics.story.outline.OutlineBase`
        """
        if issubclass(obj.__class__, OutlineBase):
            return obj.to_dict()
        if isinstance(obj, datetime.datetime):
            return obj.replace(tzinfo=datetime.timezone.utc).isoformat()
        if isinstance(obj, uuid.UUID):
            return str(obj)
        # Generalized numpy encoder/decoder -- for http transport
        # If input object is an ndarray it will be converted into a dict 
        # holding dtype, shape and the data, base64 encoded.
        if isinstance(obj, np.ndarray):
            if obj.flags['C_CONTIGUOUS']:
                obj_data = obj.data
            else:
                cont_obj = np.ascontiguousarray(obj)
                assert(cont_obj.flags['C_CONTIGUOUS'])
                obj_data = cont_obj.data
            data_b64 = base64.b64encode(obj_data).decode('utf-8')
            return dict(__ndarray__=data_b64,
                        dtype=str(obj.dtype),
                        shape=obj.shape)
        return json.JSONEncoder.default(self, obj)

def json_numpy_obj_hook(dct):
    """
    Decodes a previously encoded numpy ndarray with proper shape and dtype.
    """
    if isinstance(dct, dict) and '__ndarray__' in dct:
        data = base64.b64decode(dct['__ndarray__'].encode('utf-8'))
        return np.frombuffer(data, dct['dtype']).reshape(dct['shape'])
    return dct


def get_current_spec_version():
    """
    Get the latest version of the story outline schema via reading foldernames in 
    the schemas subfolder of this file's directory
    """
    schema_dir = os.path.join(os.path.dirname(__file__), "schemas")
    major, minor, patch = None, None, None
    latest = None
    for _dir in os.listdir(schema_dir):
        if not latest: 
            latest = semantic_version.Version(_dir)
        elif latest < semantic_version.Version(_dir):
            latest = semantic_version.Version(_dir)
    return str(latest)

def load_schema(version_number):
    path = os.path.join(os.path.dirname(__file__), "schemas", version_number, "story-outline.schema.json")
    with open(path, 'r') as f:
        data = json.load(f)
    return data

def load_latest_schema():
    version_number = get_current_spec_version()
    return load_schema(version_number)



class OutlineBase(abc.ABC):
    """
    Base object for building Story Outlines

    Includes methods for validiating, serializing, and deserializing outline classes
    """
    
    __annotations__: typing.Dict
    __required__: typing.Sequence[str]
    additional_properties: typing.Dict

    __required__ = []
    """
    A `list` of `str` objects representation attributes required to be to present
    in order for an instance to be deserialized.  Checked when the `validate` method is
    run
    """

    def __init__(self, **kwargs):
        if "additional_properties" in kwargs:
            self.additional_properties = kwargs["additional_properties"]
        else:
            self.additional_properties = kwargs

    def validate(self):
        """
        Ensures that the outline component has all required attributes.  Raises
        a `presalytics.lib.exceptions.ValidationError` if required attributes are 
        missing.  Reads the list of requried objects from the __required__ class
        variable
        """
        for key in self.__required__:
            if not hasattr(self, key):
                err_message = 'Could not load {0} object, source data missing "{1}" key'.format(self.__class__.__name__, key)
                raise ValidationError(err_message)


    @classmethod
    def deserialize(cls, json_obj: dict):
        """
        Serializes a `dict` into  class instance

        Parameters
        ----------
        json_obj : dict
            A dictionary containing attributes of the class
        
        Returns
        ----------
            A `class` instance
        """
        if type(json_obj) == cls:
            return json_obj
        updated_obj = {}
        for key, val in json_obj.items():
            new_key = to_snake_case(key)
            updated_obj.update({new_key: val})
        for req in inspect.getargspec(cls).args:
            if req not in updated_obj and req != 'self':
                updated_obj.update({req: None})
        return cls(**updated_obj)

    @classmethod
    def load(cls, json_str: str):
        """
        Serializes a `str` into  class instance

        Parameters
        ----------
        json_str : str
            A dictionary containing attributes of the class
        
        Returns
        ----------
        A `class` instance
        """
        json_obj = json.loads(json_str)
        return cls.deserialize(json_obj)

    @classmethod
    def import_yaml(cls, yaml_file: str):
        """
        Serializes a yaml from a file into a class instance

        Parameters
        ----------
        yaml_file : str
            Filepath to the file with yaml representing that class
        
        Returns
        ----------
        A `class` instance
        """
        with open(yaml_file, 'r') as file:
            obj = yaml.unsafe_load(file) # use unsafe loader per: https://github.com/yaml/pyyaml/issues/286, Monitor for fix.
        return cls.deserialize(obj)

    def export_yaml(self, filename):
        """
        Dumps yaml-formatted text representing the class instance into a file

        Parameters
        ----------
        filename : str
            Filepath to the location where the yaml fiel shoudl be dumped
        """
        with open(filename, 'w') as file:
            yaml.dump(self.to_dict(), file)

    def dump(self):
        """
        Serialize the class instance to a stringified Json object

        Returns
        ---------
        A `str` object representing the class instance in json
        """
        return json.dumps(self, cls=OutlineEncoder)

    def to_dict(self):
        """
        Converts an instance to a `dict` object of attributes.
        Used for further serialization.

        Returns
        ----------
        A `dict` object containing instance attributes
        """
        ret = {}
        for key, val in self.__dict__.items():
            if key not in self.__required__:
                if isinstance(val, list):
                    if len(val) == 0:
                        continue
                if isinstance(val, dict):
                    if len(val.items()) == 0:
                        continue
            ret_key = "{}".format(to_camel_case(key))
            ret_val = json.loads(json.dumps(val, cls=OutlineEncoder), encoding='utf-8', object_hook=json_numpy_obj_hook)
            ret[ret_key] = ret_val
        return ret


class Info(OutlineBase):
    """
    Carries metadata for a `presalytics.story.outline.StoryOutline`

    Attributes
    ----------
    revision : str
        The current revision number of the story outline
    
    date_created : datetime.datetime
        The creation date of the story in UTC 
    
    date_modified : datetime.dataime
        The last modified  date of the story in UTC 
    
    created_by : str
        The Presalytics API user id of the user that created the story
    
    modified_by : str
        The Presalytics API user id of the user that last modified the story
    
    revision_notes : str, optional
        Text explaining the changes made during that latest revision to the story
    
    """
    revision: str
    date_created: datetime.datetime
    date_modified: datetime.datetime
    created_by: str
    modified_by: str
    revision_notes: str

    def __init__(self,
                 revision,
                 date_created,
                 date_modified,
                 created_by,
                 modified_by,
                 revision_notes,
                 story_id=None,
                 **kwargs):
        super(Info, self).__init__(**kwargs)
        self.revision = revision
        self.date_created = dateutil.parser.parse(date_created).replace(tzinfo=datetime.timezone.utc)
        self.date_modified = dateutil.parser.parse(date_modified).replace(tzinfo=datetime.timezone.utc)
        self.created_by = created_by
        self.modified_by = modified_by
        self.revision_notes = revision_notes

class Plugin(OutlineBase):
    """
    Represents at plugin to be incorporated in the story during rendering process.  The information
    contain in this object is passed to subclass of `presalytics.lib.plugins.base.PluginBase` found
    in the `presalytics.PLUGINS` registry. at render-time.

    Attributes
    ----------
    kind : str
        The kind of Plugin.  Usually either "style" for a `presalytics.lib.plugins.base.StylePlugin`
        or "script" for a `presalytics.lib.plugins.base.ScriptPlugin`

    name : str
        The name of the plugin.  A name must be unique within a 
        given `presalytics.PLUGINS` registry.

    config : dict
        The configiuration of the plugin.  The keys in this dictionary are unqiue to a given 
        plugin.
    """
    kind: str
    name: str
    config: typing.Dict

    __required__ = [
        'kind',
        'config',
        'name'
    ]


    def __init__(self, kind, name, config, **kwargs):
        super(Plugin, self).__init__(**kwargs)
        self.kind = kind
        self.name = name
        self.config = config


class Widget(OutlineBase):
    """
    A represenation of an analytic or graphic object to be rendered within a 
    `presalytics.story.outline.Page`. At render-time, the `presalytics.COMPONENTS`
    registry is queried for a matching class or instance. If found, the widget is
    rendered.

    Attributes
    ----------
    kind : str
        The kind of Widget. Typically corresponds to a class that inherits from 
        `presalytics.story.components.WidgetBase`

    name : str
        The name of the Widget.  Correpsonds to a local instance of 
        `presalytics.story.components.WidgetBase` loaded into `presalytics.COMPONENTS`

    data : dict
        Widget data required so the corresponding subclass of `presalytics.story.components.WidgetBase`
        can initialize and render
        
    plugins : list of presalytics.story.outline.Plugin
        A list of plugins that must be rendered alongside this widget
    """
    name: str
    kind: str
    data: typing.Dict[str, str]
    plugins: typing.Sequence

    __required__ = [
        'name',
        'data'
    ]

    def __init__(self, name, kind, data, plugins=None, **kwargs):
        super(Widget, self).__init__(**kwargs)
        self.name = name
        self.kind = kind
        if data:
            self.data = data
        else:
            self.data = {}
        if plugins:
            self.plugins = [Plugin.deserialize(x) for x in plugins]
        else:
            self.plugins = []


class Page(OutlineBase):
    """
    A representation of a canvas on which `presalytics.story.outline.Widget` objects
    can be rendered. At render-time, the `presalytics.COMPONENTS`
    registry is queried for a matching class. If found, the page and its
    widgets are rendered.

    Attributes
    ----------
    kind : str
        The kind of Page. Typically corresponds to a class that inherits from 
        `presalytics.story.components.PageTemplateBase`

    name : str
        The name of the Page.  A name must be unique within a 
        given `presalytics.COMPONENTS` registry

    widgets : list of presalytics.story.outline.Widget
        Widgets that will be rendered along with the page        
        
    plugins : list of presalytics.story.outline.Plugin
        A list of plugins that must be rendered alongside this widget

    """
    name: str
    kind: str
    widgets: typing.Sequence[Widget]
    plugins: typing.List[Plugin]

    __required__ = [
        'name',
        'kind'
    ]

    def __init__(self, name, kind, widgets, plugins=None, **kwargs):
        super(Page, self).__init__(**kwargs)
        self.name = name
        self.kind = kind
        if widgets:
            self.widgets = [Widget.deserialize(x) for x in widgets]
        else:
            self.widgets = []
        if plugins:
            self.plugins = [Plugin.deserialize(x) for x in plugins]
        else:
            self.plugins = []


class Theme(OutlineBase):
    """
    A container for story properties that persist accross all pages of story. At render-time, 
    the `presalytics.COMPONENTS` registry is queried for a matching class. If found, the theme is
    incorporated into the story.

    Attributes
    ----------
    kind : str
        The kind of Theme. Typically corresponds to a class that inherits from 
        `presalytics.story.components.ThemeBase`

    name : str
        The name of the Theme.  Corresponds to a local instance of 
        `presalytics.story.components.ThemeBase` loaded into `presalytics.COMPONENTS`

    data : dict
        Widget data required so the corresponding subclass of `presalytics.story.components.ThemeBase`
        can initialize and render
        
    plugins : A list of presalytics.story.outline.Plugin
        A list of plugins that must be rendered alongside this widget
    """
    name: str
    kind: str
    data: typing.Dict
    plugins: typing.List[Plugin]

    __required__ = [
        'name'
    ]

    def __init__(self, name, kind, data, plugins=None, **kwargs):
        super(Theme, self).__init__(**kwargs)
        self.name = name
        self.kind = kind
        if data:
            self.data = data
        else:
            self.data = {}
        if plugins:
            self.plugins = [Plugin.deserialize(x) for x in plugins]
        else:
            self.plugins = []


class StoryOutline(OutlineBase):
    """
    A StoryOutline contains instructions for a `presalytics.story.components.Renderer` 
    (e.g., `presalytics.story.revealer.Revealer`) to render an story into html.

    A story outline's `info`, `pages`, `widgets`, and `themes` are intended to be easily editable
    by both human users and machines via json serialization and deserialization, and the ecosystem
    of tools that can be used to edit json objects. Once a valid `StoryOutline` is
    built, it contains sufficient instructions for this library find the required components in the user's
    workspace and render theme.

    StoryOutlines are stored in the [Presalytics API Story Service](https://presalytics.io/docs/api-specifications/story/).
    The story service manages version history and user permissions for StoryOutlines.  For more information
    about how outlines are used, please see the [How It Works](https://presalytics.io/docs/how-it-works/)
    section of the website.

    Attributes
    ----------

    outline_version : str
        the version of that StoryOuline schema.
    
    info : presalytics.story.outline.Info
        Metadata about this StoryOutline
    
    pages : list of presalytics.story.outline.Page
        The pages that will be rendered in this story
    
    themes : list of presalytics.story.outline.Theme
        The themes that will underlie each `presalytics.story.outline.Page` in the StoryOutline
    
    title : str, optional
        A title for the story

    description: str, optional
        A description of the story
    
    story_id : str, optional
        The Presalytics API Story Id.  Automatically added once the story outline has been pushed to
        the server.

    """
    outline_version: str
    info: Info
    pages: typing.List[Page]
    description: str
    title: str
    themes: typing.List[Theme]
    story_id: str

    __required__ = [
        'outline_version',
        'info',
        'pages',
        'title',
    ]

    def __init__(self, info, pages, description, title, themes, plugins=None, story_id="empty", **kwargs):
        super(StoryOutline, self).__init__(**kwargs)
        self.outline_version = get_current_spec_version()
        self.info = Info.deserialize(info)
        self.pages = [Page.deserialize(x) for x in pages]
        if description:
            self.description = description
        else:
            self.description = ""
        if title:
            self.title = title
        else:
            self.title = ""
        if themes:
            self.themes = [Theme.deserialize(x) for x in themes]
        else:
            self.themes = []
        if plugins:
            self.plugins = [Plugin.deserialize(x) for x in plugins]
        else:
            self.plugins = []
        self.story_id = story_id
        if not kwargs.get("validate", True):
            self.validate()

        def validate(self):
            super(StoryOutline, self).validate()
            jsonschema.validate(instance=self.to_dict(), schema=load_latest_schema())
        


import pkgutil
import importlib
import os
import typing
import logging
import importlib.util


logger = logging.getLogger(__name__)


def load_config(additional_paths: typing.List[str] = []) -> typing.Dict:
    """
    Searches the current working directory and `additional_paths` for a loadable
    module named `config.py`.  If a file is found and has a global variable named
    'PRESALYTICS', the dictionary contained in the 'PRESALYTICS' is returned.

    *Note*: A environment variable called `autodiscover_paths` is automatically
    loaded into the `additional_paths` keyword argument the the `presalytics` module
    is imported.

    Parameters
    ----------
    additional_paths : list of str, optional
        Additional filepaths to search for files named `config.py`

    Returns
    ----------
    a `dict` containing the PRESALYTICS module environment configuration

    """
    current_path = os.getcwd()
    additional_paths.append(current_path)
    config_dict = None
    try:
        for path in additional_paths:
            for name in os.listdir(path):
                if name == "config.py":
                    config_path = os.path.join(path, name)
                    config_spec = importlib.util.spec_from_file_location("config", config_path)
                    config_mod = importlib.util.module_from_spec(config_spec)
                    config_spec.loader.exec_module(config_mod) #type: ignore
                    config_dict = getattr(config_mod, "PRESALYTICS", None)
                    if not config_dict:
                        config_dict = config_mod.__dict__
                    if config_dict:
                        break
            if config_dict:
                break
        if config_dict:
            return config_dict
        else:
            return {}
    except Exception as ex:
        logger.exception(ex)
        return {}
        



# coding: utf-8

"""
    Story

    This API is the main entry point for creating, editing and publishing analytics throught the Presalytics API  # noqa: E501

    The version of the OpenAPI document: 0.3.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class CachePostRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'current_user_id': 'str',
        'nonce': 'str',
        'subdocument': 'str'
    }

    attribute_map = {
        'current_user_id': 'current_user_id',
        'nonce': 'nonce',
        'subdocument': 'subdocument'
    }

    def __init__(self, current_user_id=None, nonce=None, subdocument=None):  # noqa: E501
        """CachePostRequest - a model defined in OpenAPI"""  # noqa: E501

        self._current_user_id = None
        self._nonce = None
        self._subdocument = None
        self.discriminator = None

        if current_user_id is not None:
            self.current_user_id = current_user_id
        if nonce is not None:
            self.nonce = nonce
        if subdocument is not None:
            self.subdocument = subdocument

    @property
    def current_user_id(self):
        """Gets the current_user_id of this CachePostRequest.  # noqa: E501


        :return: The current_user_id of this CachePostRequest.  # noqa: E501
        :rtype: str
        """
        return self._current_user_id

    @current_user_id.setter
    def current_user_id(self, current_user_id):
        """Sets the current_user_id of this CachePostRequest.


        :param current_user_id: The current_user_id of this CachePostRequest.  # noqa: E501
        :type: str
        """

        self._current_user_id = current_user_id

    @property
    def nonce(self):
        """Gets the nonce of this CachePostRequest.  # noqa: E501


        :return: The nonce of this CachePostRequest.  # noqa: E501
        :rtype: str
        """
        return self._nonce

    @nonce.setter
    def nonce(self, nonce):
        """Sets the nonce of this CachePostRequest.


        :param nonce: The nonce of this CachePostRequest.  # noqa: E501
        :type: str
        """

        self._nonce = nonce

    @property
    def subdocument(self):
        """Gets the subdocument of this CachePostRequest.  # noqa: E501


        :return: The subdocument of this CachePostRequest.  # noqa: E501
        :rtype: str
        """
        return self._subdocument

    @subdocument.setter
    def subdocument(self, subdocument):
        """Sets the subdocument of this CachePostRequest.


        :param subdocument: The subdocument of this CachePostRequest.  # noqa: E501
        :type: str
        """

        self._subdocument = subdocument

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CachePostRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other




import typing
import base64
import uuid
import logging
import jinja2
import json
import os
import lxml
import lxml.html
import presalytics
import presalytics.lib.exceptions

if typing.TYPE_CHECKING:
    pass


logger = logging.getLogger(__name__)


class UrlWidget(presalytics.story.components.WidgetBase):
    """
    A `Widget` for rendering urls in an iframe.  Allows users
    load 3rd party site and applications natively within a presalytics
    story.

    Parameters
    ----------

    name : str
        the name of widget.  Must be unique within `presalytics.COMPONENTS`

    url: str
        The url to be loaded in the `<iframe>`.  For secutriy reasons, it must be https.

    """
    __component_kind__ = 'url'

    def __init__(self, 
                 name: str,
                 url: str,
                 *args,
                 **kwargs):
        self.url = url
        super(UrlWidget, self).__init__(name, *args, **kwargs)


    def to_html(self, data=None, **kwargs) -> str:
        """
        Renders url to a sandboxed iframe 
        """
        return self.create_container()

    def create_container(self, **kwargs):
        """
        Wraps the D3 objects in an endpoint at the story API load via a sandboxed `<iframe>` that
        will be rendered
        """

        empty_parent_div = lxml.html.Element("div", {
            'class': 'empty-parent bg-light',
            'style': 'height: 100%; width: 100%, display: block; text-align: left;'
        })
        frame = lxml.html.Element("iframe", {
            'src': self.url,
            'frameborder': "0",
            'scrolling': "auto",
            'class': 'd3-responsive-frame',
            'style': 'max-height: none; max-width: none; height:100%; width: 100%;',
            'sandbox': 'allow-scripts allow-forms allow-popups allow-popups-to-escape-sandbox allow-same-origin'
        })
        empty_parent_div.append(frame)
        return lxml.html.tostring(empty_parent_div).decode('utf-8')

    @classmethod
    def deserialize(cls, outline, **kwargs):
        return cls(outline.name,
                   outline.data.get('url'),
                    **kwargs)

    def serialize(self, **kwargs):
        data = {
            'url': self.url
        }
        return presalytics.story.outline.Widget(
            name=self.name,
            kind=self.__component_kind__,
            data=data,
        )

    DEFAULT_CSS = """
    html {
        height: 100%;
        width: 100%;
    }
    body {
        height: 100%;
        width: 100%;
        margin: 0px;
    }
    """
